#ifndef GODUNOV_UTILIIES_HH
#define GODUNOV_UTILIIES_HH

#include "Vector.hh"
#include "grid.hh"
#include "spatial_parameters.hh"
#include "gw_bc.hh"

// Default values for the precompiler macros

// default is no gravity
#ifndef GRAVITY_AXES
#define GRAVITY_AXES -1
#endif




class ConstantDispersion{
private:
  double default_value_;

public:
  ConstantDispersion(double _default_value)
    : default_value_(_default_value)
  {}
 
  double operator()(const size_t & , const size_t & , 
		    const Vector ,
		    const unsigned int & , 
		    const int & ) const
  {
    return default_value_;
  }

};





class PressureVelocityConverter
{
private:
  const Vector & pressure_;
  const Vector extent;
  const ConductivityCoefficients & conductivity_;
  const BoundaryConditions & boundary_conditions_;
  const double porosity_;
  mutable double max_velocity; // mutable Variablen dürfen in konstanten Methoden verändert werden!

  void check_max(double t) const
  {
    const double tabs = std::abs(t);
    if(tabs > max_velocity)
      max_velocity = tabs;
  }

public:
  PressureVelocityConverter(
							const Vector & _pressure, 
							const BoundaryConditions & _pressure_boundary_conditions,
							const Grid & grid, 
							const ConductivityCoefficients & _conductivity,
							const double _porosity
							)
	:
    pressure_(_pressure),
    extent( ( *(grid.begin()) ).extent() ),
    conductivity_(_conductivity),
    boundary_conditions_(_pressure_boundary_conditions),
    porosity_(_porosity),
    max_velocity(0)
  {}
  





  double max_tau() const
  {
    double tau=std::numeric_limits<double>::max();
    for(int i=0; i<Grid::dimensions; ++i){
      const double h = extent[i];
      const double test_tau = h/max_velocity;
      if(max_velocity==0)
	return std::numeric_limits<double>::max();
      else if(tau > test_tau)
	tau = test_tau;
    }
    return tau;
  }
  





  double operator()(const size_t & in_id, const size_t & out_id, 
		    const Vector position,
		    const unsigned int & normal_component, 
		    const int & normal_direction) const
  {
    // if(normal_component == 0)
    //   return 1e-5 * (double)normal_direction;
    // else
    //   return 0;

    const double p_in = pressure_[in_id];
    const double k_e = conductivity_(in_id);
    
    // no boundary
    if(in_id != out_id){
      const double k_f = conductivity_(out_id);
      const double p_out = pressure_[out_id];
      
      const double k = 2. / (1./k_e + 1./k_f);

      double flow = - k * ( p_out - p_in ) / extent[normal_component];
      if(normal_component==(unsigned int)GRAVITY_AXES)
	flow += - k * (double) normal_direction;

      const double velocity = flow / porosity_;

      check_max(velocity);
      return velocity;
    }
    // boundary
    else{
      const double k = conductivity_(in_id);

      double flow;
      // if dirichlet condition
      double dirichlet_value=0;
      if(boundary_conditions_.is_dirichlet(position, &dirichlet_value)){
	const double p_out = dirichlet_value;
	flow = - k * (p_out - p_in) / extent[normal_component] * 2.0 ;
	if(normal_component==(unsigned int)GRAVITY_AXES)
	  flow += - k * (double)normal_direction;
      }
      // if neumann condition
      else{
	double neumann_value = boundary_conditions_.get_neumann(position);
	flow = neumann_value;
      }
      const double velocity = flow / porosity_;

      check_max(velocity);
      return velocity;
    }
  }
};

#endif //GODUNOV_UTILIIES_HH
